// EvolveDoc.cpp : implementation file
//

#include "stdafx.h"
#include "Assignment2.h"
#include "EvolveDoc.h"
#include "MainFrm.h"
#include "Assignment2Doc.h"
#include "AntGenetic.h"
#include ".\evolvedoc.h"
#include "GODialog.h"

#define MYDEBUG 0
// EvolveDoc

IMPLEMENT_DYNCREATE(EvolveDoc, CDocument)

EvolveDoc::EvolveDoc(int popCount):
CurrentPopMember(-1), Env((Environment*) 0), PopCount(popCount),
Cols(50), Rows(50), AntNumber(10), m_PheromeneDecayRate(0.95),
EnvTime(300),generation(0),MaxDepth(11), MutProb(0.2), running(false)
{
	CrossMaxDepth = MaxDepth/2;
	SelectionSize = PopCount/3;
	createNewPop();
	makeEnvironment();

	((CMainFrame*)(AfxGetApp()->m_pMainWnd))->InfoEdit.SetWindowText("0");

	if(MYDEBUG){
		showMemLeaking(_T("EnvObjects: "), EnvObject::MemTrace(0));
		showMemLeaking(_T("CDNAStatements: "), CDNAStatement::MemTrace(0));
	}
	

}


void EvolveDoc::deletePop(){

	for(int i=0; i<Population.size();i++){
		if(Population[i])
			delete Population[i];
		Population[i] = NULL;
	}
	Population.clear();
	CurrentPopMember = -1;
}

void EvolveDoc::destroy(){
	if(Env) delete Env;
}


void EvolveDoc::makeEnvironment(){
	destroy();
	Env = new Environment(Rows, Cols, AntNumber, m_PheromeneDecayRate);
}


void EvolveDoc::createNewPop(){
	
	deletePop();
	for(int i=0; i<PopCount;i++){
		Population.push_back(new CDNAStatement());
		Population[i]->growCreate(MaxDepth);
	}
	this->UpdateAllViews(NULL);
}

void EvolveDoc::show(){
	


	if(this->CurrentPopMember >= 0){
		running = false;
		CAntGenetic p( Population[this->CurrentPopMember]);
		p.DoModal();
		Population[this->CurrentPopMember] = p.getDNA();
		this->UpdateAllViews(NULL);
	}
	else{

		AfxMessageBox("Pls select an individual");
	}

}


BOOL EvolveDoc::OnNewDocument()
{
	if (!CDocument::OnNewDocument())
		return FALSE;
	return TRUE;
}

EvolveDoc::~EvolveDoc()
{
	
	if(Env) delete Env;
	for(int i=0; i<this->Population.size();i++)
		delete Population[i];
	Population.clear();



	if(MYDEBUG){
		showMemLeaking(_T("EnvObjects: "), EnvObject::MemTrace(0));
		showMemLeaking(_T("CDNAStatements: "), CDNAStatement::MemTrace(0));
	}
}


BEGIN_MESSAGE_MAP(EvolveDoc, CDocument)
	ON_COMMAND(ID_VIEW_SETTINGS, OnViewSettings)
	ON_COMMAND(ID_STOP, OnStop)
	ON_COMMAND(ID_GO, EvolveBastard)
	ON_COMMAND(PLAYCDNA, PlayTheCDNA)
END_MESSAGE_MAP()


// EvolveDoc diagnostics

#ifdef _DEBUG
void EvolveDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void EvolveDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG


// EvolveDoc serialization

void EvolveDoc::Serialize(CArchive& ar)
{
	if(CurrentPopMember >= 0){
		CAssignment2Doc Doc(Population[CurrentPopMember]) ;
		Doc.Serialize(ar);
	}else
	AfxMessageBox("No individual has been selected");
}


// EvolveDoc commands


void EvolveDoc::userMessage1(){
	AfxMessageBox(_T("You must select an individual first"));
}


void EvolveDoc::OnStop(){
	if(running){
		CString Pre, Full;
		Pre.Format("%d", this->generation);
		Full = _T("Stopping run on generation: ") + Pre;
		AfxMessageBox(Full.GetBuffer());
		running = false;
			
	}else
		AfxMessageBox(_T("Not running"));
}

double EvolveDoc::grade(CDNAStatement* Stat){
	
	MSG msg;
	while(::PeekMessage(&msg, 0, 0, 0, PM_REMOVE)){

		if(!AfxGetApp()->PreTranslateMessage(&msg))
			{
				::TranslateMessage(&msg);
				::DispatchMessage(&msg);
			}
	}
	AfxGetApp()->OnIdle(0);
	AfxGetApp()->OnIdle(1);

	if(running){
		makeEnvironment();
		return Env->evalCDNA(*Stat, EnvTime);
	}else
		return -1.0;
}

BOOL EvolveDoc::OnOpenDocument(LPCTSTR lpszPathName)
{
	return TRUE;
}

void EvolveDoc::Select(){
	

	vector<CDNAStatement*> NewPopulation;		
	
	int TotFit = (int)this->getTotalFitness();
	if(TotFit < 1) TotFit=1;

	int ind = 0; 
	double acc;
	
	ASSERT((SelectionSize < Population.size())&&(SelectionSize > 0));
	
	while(NewPopulation.size() < SelectionSize){
		acc = (double)((rand()%TotFit) + 1);
		while ((acc - this->Population[ind]->fitness) > 0.0){
				acc -= this->Population[ind]->fitness;
				ind = (ind+1)%this->Population.size();
		}
		NewPopulation.push_back(new CDNAStatement(*Population[ind]));
	}
	

	
	for(int i=0; i<Population.size();i++){
		if(Population[i]) {
			delete Population[i];
			Population[i] = NULL;
		}
	}

	for(int i=0; i<NewPopulation.size(); i++)
		Population[i] = NewPopulation[i];
	
	this->UpdateAllViews(NULL);
}



void EvolveDoc::Breed(){

		int indMom = 0; 
		int indPop = 0;
		double acc;
		int TotFit = (int)this->getTotalFitness();
		if(TotFit < 1) TotFit=1;


		int CurrentPopSize = 0;
		
		for(int i=0; i<this->Population.size(); i++)
			if(this->Population[i])
				CurrentPopSize++;
		int BegPopSize = CurrentPopSize;

		while(CurrentPopSize < this->PopCount){
				acc = (double)((rand()%TotFit)+1);

				while ((acc - this->Population[indMom]->fitness) > 0.0){
					acc -= this->Population[indMom]->fitness;
					indMom = rand()%BegPopSize;
				}

				Population[indMom]->setCrossOverMaxDepth(CrossMaxDepth);
				Population[indPop]->setCrossOverMaxDepth(CrossMaxDepth);
				Population[indPop]->setMutationProb(MutProb);

				this->Population[CurrentPopSize]
						= &((*(this->Population[indPop]))*(*(this->Population[indMom])));
				
				CDNAStatement* Kid =  this->Population[CurrentPopSize];
				indPop = (indPop+1)%BegPopSize;
				CurrentPopSize++;
				//TotFit = (int)this->getTotalFitness();
		}
		this->UpdateAllViews(NULL);
}


void EvolveDoc::EvaluateAll(){
	double fit;
	((CMainFrame*)(AfxGetApp()->m_pMainWnd))->Progress.SetRange(0, (Population.size())/10);
	((CMainFrame*)(AfxGetApp()->m_pMainWnd))->Progress.SetStep(1);



	COUNTER i = 0;
	while ((i < Population.size())&&(running)){
		if(i%10 == 0) ((CMainFrame*)(AfxGetApp()->m_pMainWnd))->Progress.StepIt();
		fit = this->grade(Population[i]);
	
		if(fit >= 0) Population[i]->fitness = fit;
		
		i++;
	}

	this->UpdateAllViews(NULL);
}



void EvolveDoc::EvolveBastard(){

	if(running){
		CString g;
		g.Format("already running : current generation %d", generation);
		AfxMessageBox(g);
	}
	else{
		CString t;
		((CMainFrame*)(AfxGetApp()->m_pMainWnd))->InfoEdit.GetWindowText(t);
		stringstream trad1;
		trad1<<(LPCTSTR) t;
		int evolveTo;
		running = true;
		trad1>>evolveTo;
		t.Format("%d", generation);
		((CMainFrame*)(AfxGetApp()->m_pMainWnd))->InfoEdit.SetWindowText(t.GetBuffer());


		while((evolveTo > generation)&&(running)){
			EvaluateAll();
			
			if (running){
				Select();
				Breed();
				generation++;
			}

			t.Format("%d", generation);
			((CMainFrame*)(AfxGetApp()->m_pMainWnd))->InfoEdit.SetWindowText(t.GetBuffer());
		}
		running = false;
		CString Msg = _T("Evolution run finished normally at generation ");
		Msg+=t;
		AfxMessageBox(Msg.GetBuffer());
		
	}
}

void EvolveDoc::OnViewSettings(){


	running = false;
	CInfoSetting T(NULL, Rows, Cols, AntNumber, 
		this->PopCount, this->SelectionSize, this->MutProb, this->MaxDepth, this->EnvTime, this->CrossMaxDepth);
	
	T.DoModal();
	Rows = T.Rows;
	Cols = T.Cols;
	AntNumber = T.AntNum;


	if(T.PopCount) PopCount=T.PopCount;
	this->MutProb = T.MutRate;
	if(T.MaxDepth>1)	MaxDepth = T.MaxDepth;
	if(T.TimeClicks > 10) this->EnvTime = T.TimeClicks;
	if(T.CrossDepth < MaxDepth )CrossMaxDepth = T.CrossDepth;
	if(T.SelectionSize<PopCount)SelectionSize=T.SelectionSize;
	createNewPop();

}

void EvolveDoc::PlayTheCDNA(){
	
	if(this->CurrentPopMember >= 0){
		running = false;
		CMultiDocTemplate* NewTemp = ((CAssignment2App*)AfxGetApp())->pDocTemplate1;
		CAssignment2Doc* NewDoc =  new CAssignment2Doc(Population[this->CurrentPopMember]);
		CFrameWnd* pFrame = NewTemp->CreateNewFrame(NewDoc, NULL);
		NewTemp->InitialUpdateFrame(pFrame, NewDoc, true);
	}
	else	userMessage1();
	
}














