// Pheromone.h: interface for the Pheromone class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_PHEROMONE_H__6E81BE89_A79D_451F_9ABD_5EEAD07C7C89__INCLUDED_)
#define AFX_PHEROMONE_H__6E81BE89_A79D_451F_9ABD_5EEAD07C7C89__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000
#include "StdAfx.h"

#define HILLPHEROMONETYPE (unsigned char)1
#define XPLOPHEROMONETYPE (unsigned char)2
#define FOODPHEROMONETYPE (unsigned char)3

#define HILLPHEROMONESTRENGTH (double)3.0f
#define XPLOPHEROMONESTRENGTH (double)3.0f
#define FOODPHEROMONESTRENGTH (double)3.0f

#define HILLPHEROMONEDECAY (double)0.5f
#define XPLOPHEROMONEDECAY (double)0.5f




class Pheromone : public EnvObject {

protected:

	
    double m_PheromeneDecayRate;
    double MaxStrength;
    double Strength;
    unsigned char Type;
    
   
public:
	
	//static void setPheromoneDecay(double Deca){Pheromone::m_FoodPheromoneDecay = Deca;};
	Pheromone(int X, int Y, unsigned char type, double = 0.5);
	Pheromone(const Pheromone& P);			
	virtual ~Pheromone();
    
    void decay(){
        if(isAntHillPheromone())     {Strength *= HILLPHEROMONEDECAY; return;}
        if(isExplorePheromone())     {Strength *= XPLOPHEROMONEDECAY; return;}
		if(isFoodCarryingPheromone()){Strength *= m_PheromeneDecayRate; return;}
    }
    
    double getStrength(){return Strength;};
    
    void refreshPheromone(){
        if(isAntHillPheromone())     {Strength += HILLPHEROMONESTRENGTH; return;}
        if(isExplorePheromone())     {Strength += XPLOPHEROMONESTRENGTH; return;}
        if(isFoodCarryingPheromone()){Strength += FOODPHEROMONESTRENGTH; return;}
        throw "Unknown Pheromone Type at Pheromone::refreshPheromone()";
    };
    
    
    bool isAntHillPheromone()       {return Type == HILLPHEROMONETYPE;};
    bool isExplorePheromone()       {return Type == XPLOPHEROMONETYPE;};
    bool isFoodCarryingPheromone()  {return Type == FOODPHEROMONETYPE;};
    bool isSameType(const Pheromone& P){return P.Type == Type;};
    unsigned char getPheromoneType(){return Type;};    
    
    void draw(GLfloat hor_step, GLfloat vert_step);
    ObjectType objectType(){return PHEROMONE;};
};



#endif // !defined(AFX_PHEROMONE_H__6E81BE89_A79D_451F_9ABD_5EEAD07C7C89__INCLUDED_)
