// Environment.h: interface for the Environment class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_ENVIRONMENT_H__9D7BFE99_0B20_4BC1_985E_F68B1D203E70__INCLUDED_)
#define AFX_ENVIRONMENT_H__9D7BFE99_0B20_4BC1_985E_F68B1D203E70__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000



#include "StdAfx.h"


class Environment : public CObject{
    
    protected:
        EnvObject**** EnvMap;


        vector<Pheromone*> Pheromones;
        vector<Ant*> Ants;
        vector<FoodParticule*> FoodParticules;
	
	AntHill* Hill;
        
        int EnvThingsSize;
        int EnvAntsNum;
	
        int foodNum;
	int Rows;
        int Cols;
        double m_PheromeneDecayRate;
	

        bool addAnt(Ant* A);
        bool addFood(FoodParticule* F);   
        bool addPheromone(Pheromone* P);
        void addToPheromoneArray(Pheromone* P);
        bool addAntHill(AntHill* P);
        
        void removeEnvObject(EnvObject* o){
		EnvObject* temp = o;
		int he = 0;
		if(o){
			
			for(int k=0; k<MAXOBJPERMAPSQUARE; k++)
			if(EnvMap[o->getX()][o->getY()][k] == o){
				EnvMap[o->getX()][o->getY()][k] = (EnvObject*)0;
				if(isFood(o)) foodNum--;
				delete o;
				EnvThingsSize--;
				return;
		    }
	    }
             throw "Trying to remove an inexistant object at Environment::removeEnvObject";
	};
        
        void updatePheromones(){
            for(int i=0; i<Pheromones.size(); i++)
                if(Pheromones[i]){
                    Pheromones[i]->decay();
                    if(Pheromones[i]->getStrength()<PHEROMONETOL){
                        removeEnvObject(Pheromones[i]);
                        Pheromones[i] = (Pheromone*) 0;               
                    }                    
                }
        };


        void updateAnts(const CDNAStatement& Instructions){
			for(int i=0; i<Ants.size();i++)
				try {
					Instructions.Eval(Ants[i]);
				}
				catch (LPTSTR ErrMessage){
					throw ErrMessage;
					break;
				}
		};

        void updateHill(){
             for(int i=0; i<MAXOBJPERMAPSQUARE; i++)
				 if(isFood(EnvMap[Hill->getX()][Hill->getY()][i])&&(!((FoodParticule*)(EnvMap[Hill->getX()][Hill->getY()][i]))->isBeingCarried))
                    removeEnvObject(EnvMap[Hill->getX()][Hill->getY()][i]);
                    
         };
        
        void Destroy();    
        bool canGoTo(int x, int y){
            if(!inBounds(x, y)) return false;
            for(int k=0; k<MAXOBJPERMAPSQUARE;k++)
                    if(!EnvMap[x][y][k]) return true;
            return false;        
        }
        bool inBounds(int x, int y){
            if((x<0)||(x>=Rows)) return false;
            if((y<0)||(y>=Cols)) return false;
            return true;
        }
        
	int WrapCols(int NewTY) {return (NewTY >=0 )?NewTY%Cols:Cols+NewTY;};
	int WrapRows(int NewTX) { return (NewTX >=0 )?NewTX%Rows:Rows+NewTX;};
    public:
	
	bool viewPheromone;
	int getTotalFood(){return foodNum;};
	unsigned int getRows(){return Rows;};
	unsigned int getCols(){return Cols;};


	double foodPheromoneStrengthHere(int x, int y);
        
        
        bool isAntHill(EnvObject* o){
            if(o) return o->objectType() == ANTHILL;
            return false;
        };
        bool isFood(EnvObject* o){
            if(o) return o->objectType() == FOOD;
            return false;
        };
        bool isAnt(EnvObject* o){
            if(o) return o->objectType() == ANT;
            return false;
        };


        bool Environment::isPheromone(EnvObject* o){
            if(o) return o->objectType() == PHEROMONE;
            return false;
        };
        bool Environment::isFoodPheromone(EnvObject* o){
            if(o) if ((isPheromone(o))&&(((Pheromone*)o)->isFoodCarryingPheromone())) return true; 
            return false;   
        };

        Orientation DirectionOfAntHill(int Posi, int Posj);
        FoodParticule* foodHere(int x, int y);        
        bool moveObject(EnvObject* o, int NewX, int NewY);
        bool addEnvObject(EnvObject* o);
        void draw(GLfloat GridWidth, GLfloat GridHeight);
        
        void timeTick(const CDNAStatement& Instructions){
            
            //Adding ants to population if we still
            //don't have the right number
            if(Ants.size()<EnvAntsNum)
                addEnvObject( new Ant(Hill->getX(),Hill->getY(), this, NORTH, m_PheromeneDecayRate));
            updateAnts(Instructions);      
            updateHill();
            updatePheromones();

        };
	

	void reset(){

		
		foodNum = 0;
		int foodctr1;
		int foodctr2;
		for(foodctr1 = Cols/4; foodctr1<((2*Cols)/3)+rand()%Cols; foodctr1++)
		for(foodctr2 = rand()%2; foodctr2<(rand()%7); foodctr2++)
			addEnvObject( new FoodParticule(foodctr1, foodctr2));


		for(foodctr1 = 4; foodctr1<4+rand()%Rows; foodctr1++)
			for(foodctr2 = 10+rand()%4; foodctr2<17+rand()%(Cols/3); foodctr2++)
				addEnvObject( new FoodParticule(foodctr1, foodctr2));
		
		while(!addEnvObject( new AntHill(rand()%Rows, rand()%Cols)));

	};

	double evalCDNA(const CDNAStatement& Instructions, int EvalTime){
	
					double currentFood = (double)getTotalFood();
					for(int i=0; i<EvalTime;i++){
						try{
							timeTick(Instructions);
						}
						catch(LPTSTR ErrMessage){
							AfxMessageBox(ErrMessage);
						}
					}
					//reward for getting food
					
					return ((currentFood-(double)getTotalFood())/currentFood) * 15.0f;
	};
        
        Environment(int Rows, int Cols, int AntNum, double PheromeneDecayRate=0.5);
        virtual ~Environment();
};
#endif // !defined(AFX_ENVIRONMENT_H__9D7BFE99_0B20_4BC1_985E_F68B1D203E70__INCLUDED_)
